import copy
import random
import time


class Preteky():
    '''Trieda pre generovanie zaznamov z orientacneho behu'''
    def __init__(self, pocet_kontrol=None, pocet_pretekarov=None):
        '''Nastavenie parametrov behu.

        :param pocet_kontrol: pocet kontrol
        :type pocet_kontrol: int
        :param pocet_pretekarov: pocet pretekarov
        :type pocet_pretekarov: int
        '''
        self.__pridaj_pred = 0.25
        self.__pridaj_za = 0.25
        self.__pridaj_do = 0.25
        self.__zmaz_z = 0.25

        if pocet_pretekarov is None:
            self.__nastav_pocet_pretekarov(random.randrange(1, 21))
        else:
            self.__nastav_pocet_pretekarov(pocet_pretekarov)

        if pocet_kontrol is None:
            self.__nastav_pocet_kontrol(random.randrange(2, 50))
        else:
            self.__nastav_pocet_kontrol(pocet_kontrol)

        self.__generuj_zaznamy_pretekov()

    def __generuj_zaznamy_pretekara(self, meno):
        '''Genruje zaznamy pretekara s danym menom/cislom

        :param meno: meno/cislo pretekara
        :type meno: int or str
        :return: zaznamy pretekara
        :rtype: list[list[int, int or str, int]]
        '''
        kontroly = [i for i in range(1, self.pocet_kontrol)]
        if len(kontroly) > 1 and random.random() < self.__pridaj_do:
            kontroly.insert(random.randrange(1, len(kontroly)),
                            random.randrange(0, self.pocet_kontrol))
        if random.random() < self.__zmaz_z:
            kontroly.pop(random.randrange(0, len(kontroly)))
        if random.random() < self.__pridaj_pred:
            kontroly.insert(0, random.randrange(0, self.pocet_kontrol))
        if random.random() < self.__pridaj_za:
            kontroly.insert(self.pocet_kontrol,
                            random.randrange(0, self.pocet_kontrol))
        kontroly.insert(0, 0)
        zaznamy_pretekara = []
        cas = int(time.time()) - 10000 + random.randint(0, 100)
        zaznamy_pretekara.append([kontroly[0], meno, cas])
        for i in range(1, len(kontroly)):
            cas = cas + abs(kontroly[i] - kontroly[i - 1]) * 100 + random.randrange(0, 10 * (abs(kontroly[i] - kontroly[i - 1]) + 1))
            zaznamy_pretekara.append([kontroly[i], meno, cas])
        return zaznamy_pretekara

    def __vrat_pocet_kontrol(self):
        '''Vrati pocet kontrol v preteku

        :return: pocet kontrol
        :rtype: int
        '''
        return self.__kontroly

    def __nastav_pocet_kontrol(self, pocet_kontrol):
        '''Nastavi pocet kontrol preteku

        :param pocet_kontrol: pocet kontrol
        :type pocet_kontrol: int
        :raises TypeError: ak pocet nie je cele cislo
        :raises ValueError: ak pocet je mensi ako 2
        :rtype:
        '''
        if not isinstance(pocet_kontrol, int):
            raise TypeError('Chyba! Počet kontrol musí byť celé čislo.')
        if pocet_kontrol < 2:
            raise ValueError('Chyba! Počet kontrol musí byť aspoň 2.')
        self.__kontroly = int(pocet_kontrol)

    pocet_kontrol = property(__vrat_pocet_kontrol)

    def __vrat_pocet_pretekarov(self):
        '''Vrati pocet pretekarov

        :return: pocet pretekarov
        :rtype: int
        '''
        return self.__pretekarov

    def __nastav_pocet_pretekarov(self, pocet_pretekarov):
        '''Nastavi pocet pretekarov preteku

        :param pocet_pretekarov: pocet pretekarov
        :type pocet_pretekarov: int
        :raises TypeError: ak pocet nie je cele cislo
        :raises ValueError: ak pocet je mensi ako 1
        :rtype:
        '''
        if not isinstance(pocet_pretekarov, int):
            raise TypeError('Chyba! Počet pretekarov musí byť celé čislo.')
        if pocet_pretekarov < 1:
            raise ValueError('Chyba! Počet pretekárov musí byť aspoň 1.')
        self.__pretekarov = int(pocet_pretekarov)

    pocet_pretekarov = property(__vrat_pocet_pretekarov)

    def __generuj_zaznamy_pretekov(self):
        '''Generuje zaznamy preteku'''
        self.__zaznamy_pretekov = []
        for i in range(self.pocet_pretekarov):
            self.__zaznamy_pretekov += self.__generuj_zaznamy_pretekara(i)
        random.shuffle(self.__zaznamy_pretekov)

    def __vrat_zaznamy_pretekov(self):
        '''Vrati zaznamy preteku

        :return: zaznamy preteku
        :rtype: list[list[int, int or str, int]]
        '''
        return copy.deepcopy(self.__zaznamy_pretekov)

    zaznamy_pretekov = property(__vrat_zaznamy_pretekov)


'''
Pouzitie generatora zaznamov pretekov: Preteky(pocet_kontrol, pocet_pretekarov)
'''
preteky = Preteky(5, 3)
pocet_pretekarov = preteky.pocet_pretekarov
pocet_kontrol = preteky.pocet_kontrol
zaznamy_pretekov = preteky.zaznamy_pretekov

'''
Vytvor funkciu vysledkova_listina() s vhodnými parametrami.
'''
